// 2006-09-07 started refactoring the SudokuSingleFile.cpp
#ifndef SUDOKU_H_
#define SUDOKU_H_

#include <time.h>  //time_t

#include <FL/Fl.H>
#include <FL/Fl_Window.H> //Fl_Window
#include <FL/Fl_Sys_Menu_Bar.H>  //Fl_Sys_Menu_Bar
class Fl_Group;
class Fl_Help_Dialog;  //Fl_Help_Dialog 
class Fl_Preferences;  //Fl_Preferences

class SudokuCell;
class SudokuSound;

//version numbers
#define VERSION	"2beta4"
#define BUILD	"0"

//
// Default sizes...
//

#define GROUP_SIZE	160
#define CELL_SIZE	50
#define CELL_OFFSET	5

//TODO find a nice deault sudoku (not so hard)
#define DEFAULTPUZZLE ".1......9...3..8........6......124..7.3......5........8..6.........4..2....7...5."

#ifdef __APPLE__
#  define MENU_OFFSET	0
#else
#  define MENU_OFFSET	25
#endif // __APPLE__

class Sudoku : public Fl_Window 
{
  SudokuSound	    *sound_;

  Fl_Sys_Menu_Bar *menubar_;
  Fl_Group	      *grid_;
  Fl_Group	      *grid_groups_[3][3];
  
  time_t	        seed_;
  int		          difficulty_;  //0..3;
  char		        grid_values_[9][9];  //contains the solved puzzle
  SudokuCell	    *grid_cells_[9][9]; //contains the game
  
  int  hassound_; //1 = sound; 0 sound is off
  int  autohints_; //1 = on; 0 is off
  int  hassolution_; //1 has solution (original difficulty); 0= initially for imported files
  int  imported_; //0 generated (diff 0..3); 1= imported files => hassolution_ => 0; until one is found
  int  autounique_; //1 = on; 0 is off
  int  promoteunique_; //1=0 0 is off
  char cellHint[9][9];  //keep single hint of cell, showuniquehints() & testcell()
                        //-1 inited
  char testGrid[9][9][11];  // i j values; [0] the actual value, [1.9] hints, [10] 1:value/0:hints 
  int  lastFocusI, lastFocusJ;  //position of last cell having the focus [row][column]
  long solvechecks;  //number of checks done to find the number of possible solutions
  int  solutionsStopped;  //to break of a path of possible solutions if check is not ok

  //callback routines (are static)
  static void	check_cb(Fl_Widget *widget, void *);
  static void	close_cb(Fl_Widget *widget, void *);
  static void	diff_cb(Fl_Widget *widget, void *d);
  static void	help_cb(Fl_Widget *, void *);
  static void	new_cb(Fl_Widget *widget, void *);
  static void	cell_callback(Fl_Widget *widget, void *);
  static void	restart_cb(Fl_Widget *widget, void *);
         void	set_title();
  static void	showSolution_cb(Fl_Widget *widget, void *);
  //
  static Fl_Help_Dialog *help_dialog_;
  static Fl_Preferences	prefs_;
  //PLJ:
  static void	input_cb(Fl_Widget *widget, void *); //(import a string todo a file)
  static void	savepuzzle_cb(Fl_Widget *widget, void *);
  static void	savegame_cb(Fl_Widget *widget, void *);
  static void	loadgame_cb(Fl_Widget *widget, void *);
  static void	hassound_cb(Fl_Widget *widget, void *);
  static void	autohints_cb(Fl_Widget *widget, void *);
  static void	hints_cb(Fl_Widget *widget, void *);
  static void	hintthis_cb(Fl_Widget *widget, void *);
  static void	updatehintsfromthis_cb(Fl_Widget *widget, void *);
  static void	showuniquehints_cb(Fl_Widget *widget, void *);
  static void	autounique_cb(Fl_Widget *widget, void *);
  static void	promoteunique_cb(Fl_Widget *widget, void *);
  static void	autopromoteunique_cb(Fl_Widget *widget, void *);
  static void	removehints_cb(Fl_Widget *widget, void *);
  static void	showthisvalue_cb(Fl_Widget *widget, void *);
  static void	helpme_cb(Fl_Widget *widget, void *);
  static void	NumberOfSolutions_cb(Fl_Widget *widget, void *);
  static void	removebckgrnds_cb(Fl_Widget *widget, void *);
  static void	printgame_cb(Fl_Widget *widget, void *);
  bool import_stopped; //import string walker guard
  
public:
  Sudoku();
  virtual ~Sudoku();
	
  void 		check_game(bool highlight);
  void		load_pref_game();
  void		new_game(time_t seed);
//  int		  next_value(SudokuCell *c);
  int		  next_value(int row, int col);
  void		resize(int X, int Y, int W, int H);
  void		save_pref_game();
  void		showSolution_game();
  //PLJ:
  void    input_game(char * import);
  int     getNextValue(char* importString, int teller);
  void    setFocusCell(int I,int J);
  void    updatehintsfromthis(int i, int j);
  void    hintthiscell(int i, int j);
  int     getautohints();
  void    showuniquehints();
  void    testcell(SudokuCell* cell, int* hints, int index);
  void    printHints(int * hints);
  int     getautounique();
  int     getpromoteunique();
  void    hints();
  void    promoteuniquehints();
  void    removehints();
  void    showthisvalue(int i, int j);
  void	  cleanBackgrounds();
  void    helpme();
  void    numberOfSolutions();
  void    solveGrid(int startValue, int* solutions);
  int     checkGrid();
  void    printGridcell(int printi, int printj);
  void    printGrid();
  void    printGame(char* filename);
};
#endif /*SUDOKU_H_*/
