// Enhanced Sudoku game using the Fast Light Tool Kit (FLTK).
//
// extended version Copyright Paul Janson & Rob Golsteijn 2006
// Copyright 2005-2006 by Michael Sweet.
// And modifications by John T. Haller of PortableApps.com to improve
// portability. (version 1.1.7a) 
// We took Fl_Preferences.cxx and copied a few Sudoku.cxx changes.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems on the following page:
//
//     http://www.p-l-j.org/Sudoku/
//
#include "Sudoku.h"

int
main(int argc, char *argv[])
{
  Sudoku s;

  // Show the game...
  s.show(argc, argv);

  // Load the previous game...
  s.load_pref_game();

  // Run until the user quits...
  return (Fl::run());
}
// FLTK example by Michael Sweet
// Copyright 2005-2006 by Michael Sweet.
//
// extended version Paul Janson & Rob Golsteijn
//
// Single file
// 2006-08-30 import and continue on the games
// 2006-08-31 import from difficulty to sudoku pref
//            switch of sound
//            ordered hints
//            auto hints
//            ....todo
//2006-09-01  show uniques
//            promote unique
//            export puzzle
//            save status/game
//2006-09-04  update show unique  (include values given and user values)
//            load status/game
//            if delete value & autohints_ => DO IT
//            remove hints
//            updated focus event
//            show this value '?'
//            remove colors ' '
//2006-09-05  backtrack possible solutions
//2006-09-06  refined backtracking
//            standalone project with makefile
//
//2006-09-07  refactoring
//2006-09-13  refactor redraw of cells & redraw of window
//            add imported_ (next to hassolution_) & store a found solution
//              for imported games after find number of solutions
//
//2006-09-19  fixes PRs after 2.0beta3-b0 release
//            3 check routine YELLOW sometimes shows and sometime doesn't=>
//              check when completely filled in and show yellows if not correct
//            5 redraw after restart =>
//              just slow, due to sound, redraw only complete after all updates
//              added changed cell redraw duriong sound, is animation
//            2 Menu name differs from dialog name : save puzzle / export puzzle
//            1 Auto hints not executed immediately after an import (yet) =>
//              added all option checks :)
//            4 Double click only uses the box for determining the next value, 
//              add row/col => update sudoku::next_value
//            added save a printable game functions
//2006-09-22 2.0beta4-b0 Merge Portable Apps changes Mainly Fl_Preferences.cxx 
//           So from now, the new FLTK libs are the portableApps version 
//           of the FLTK libs!
//
// DONE & TODO:
// V toggle for sound off ;^) speeds up animations to ;^)
// V move imported from difficulty to its own preference/bool/int "imported_"
// V export puzzel <files?>
// V save work (puzzel, plus choizen, plus hints)
// V load work (if 1 row only the puzzle ;^)
// V keep hints at a fixed place 123,456,789 for easy overview
// V generate hints (for all empty places) using row column and box
// V (open cell) generate hints (for THIS cell) [row, col,box]
//   <who is this i,j>
// V (value cell) update hints from this cell value [row,col,box]
// V auto update hints on value setting
// V highlight unique in row/col/box  (auto highlight function)
// V promote unique hint to value
// V auto unique & promote
// V if delete value & autohints_ => DO IT show hints for this cell again
// V remove hints (kind of RESTART without removing the USER values)
// ? what about resetting test_values when entering a value, keep as rollback,
//   and not draw for user value?
//   maybe outdated idea, because of autohint update when '0' or delete.
//   kind of UNDO function
// - Highlight cells CYAN from Auto promote unique to value
// - if Auto promote do a unique (recursive till not possible anymore??
//   & highlight promoted cells CYAN
// V highlight all with this value (temporary) MAGENTA
// v When to remove color of unique hints (cyan backgrounds) & black hints?
//   When to remove color of this value   (magenta backgrounds) ?
//   => SPACEBAR & remove backgrounds :)
// V Help me : computer give a HINT (if hassolution), random on empty fields
//    highlight GREEN
// ? What to do with a helpme in a multi solution puzzle??
// V check how many solutions (backtracking)
//   v store 1st solution if imported & hassolution_ is false [option]
//   v breakoff checks in backtracking if check is not OK, when to check?
//   - improve initial hints with unique promote until finished.
// V update redraw strategy not in each cell access but after each callback method only
//   V fix newgame animation based on sound delay and updates, in new_game
// V Updated check game for box checks
// > we deliberatly "screwed up" the all cells readonly feature in case of solved

// = useful UI names and shortcuts
// - UI stuf grayout if option unavailable
// - radiobuttons none when imported???
// - has solution in import title (FLTK Status bar??)
// - calculate dificulty of imported game and in title (number of readonly cells)
// - numbe of backtrackings done until stopped in feedback
// - autocheck: toggle on/off to check after each user update!
//
// X print to printer (no FLTK support) NOT until supported by toolkit
// v print to file puzzle / game (how to selct these what sze printing)
//   - print puzzle only
//   - print puzzle with all 9 hints
//   - print puzzle with an hints() hints (row/col/box)
//   v print game as user has it on the screen

// TODO v2.0:
//           +/- bugfixed
//           v printable files
//           v merge portable app extentions/changes 
//             (push !readonly / preferences in fltk / help txt)
//           - small features
//           makefile for the zip files 
//           spellcheck website
//           make consistent screenshots for website

// - inifiles to configure the printing symbols to be used for the
//   offline game saving/printing.

// - Create a unique puzzle support: promote user value to puzzle value (readonly)
// ? what about hints in user entered value cells?
// - when deleting a user value: do we need to ADD hints to all row/col/box for the value removed??
// - store solution in Calculate number of solutions as a option [v]
// - configure the number of solutions before stopping (now 2)
// - double click not increment if value but next possible value
// - autopromote -> do a check afterwards

// - FAMOUS LAST : solve really
//   -> generate real sudoku(only 1 solution)
//   - log automatic solving actions (unique in row, promote single in row,...).

// save the logs in file
// save the solutions found in file
